@@ -5,15 +5,114 @@
 # This code is part of the package:
 # Antarctic-plots (https://github.com/mdtanker/antarctic_plots)
 #
+import glob
 import os
+from getpass import getpass
+from pathlib import Path
 
 import pandas as pd
 import pooch
 import pygmt
+import requests
 import xarray as xr
 from pyproj import Transformer
 
-from antarctic_plots import utils
+from antarctic_plots import regions
+
+
+def resample_grid(
+    grid,
+    initial_spacing,
+    initial_region,
+    spacing=None,
+    region=None,
+):
+
+    if spacing is None:
+        spacing = initial_spacing
+    if region is None:
+        region = initial_region
+
+    if (spacing == initial_spacing) and (region == initial_region):
+        # print('spacing and region same as original, no processing')
+        resampled = grid
+
+    elif (spacing == initial_spacing) and (region != initial_region):
+        # print('spacing same as original, extracting new region')
+        resampled = pygmt.grdcut(
+            grid=grid,
+            region=region,
+            # extend=True,
+            # verbose="q",
+        )
+
+    elif spacing < initial_spacing:
+        # print('spacing smaller than original, resampling')
+        resampled = pygmt.grdsample(
+            grid=grid, spacing=spacing, region=region, verbose="q"
+        )
+
+    elif spacing > initial_spacing:
+        # print('spacing larger than original, filtering and resampling')
+        filtered = pygmt.grdfilter(
+            grid=grid,
+            filter=f"g{spacing}",
+            spacing=spacing,
+            # region=region,
+            distance="0",
+            nans="r",
+            # verbose="q",
+        )
+        resampled = pygmt.grdcut(
+            grid=filtered,  # sampled,
+            region=region,
+            extend=True
+            # verbose="q",
+        )
+
+    return resampled
+
+
+class EarthDataDownloader:
+    """
+    Adapted from IcePack: https://github.com/icepack/icepack/blob/master/icepack/datasets.py  # noqa
+    Either pulls login details from pre-set environment variables, or prompts user to
+    input username and password.
+    """
+
+    def __init__(self):
+        self._username = None
+        self._password = None
+
+    def _get_credentials(self):
+        if self._username is None:
+            username_env = os.environ.get("EARTHDATA_USERNAME")
+            if username_env is None:
+                self._username = input("EarthData username: ")
+            else:
+                self._username = username_env
+
+        if self._password is None:
+            password_env = os.environ.get("EARTHDATA_PASSWORD")
+            if password_env is None:
+                self._password = getpass("EarthData password: ")
+            else:
+                self._password = password_env
+
+        return self._username, self._password
+
+    def __call__(self, url, output_file, dataset):
+        auth = self._get_credentials()
+        downloader = pooch.HTTPDownloader(auth=auth, progressbar=True)
+        try:
+            login = requests.get(url)
+            downloader(login.url, output_file, dataset)
+        except requests.exceptions.HTTPError as error:
+            if "Unauthorized" in str(error):
+                pooch.get_logger().error("Wrong username/password!")
+                self._username = None
+                self._password = None
+            raise error
 
 
 def sample_shp(name: str) -> str:
@@ -40,10 +139,134 @@ def sample_shp(name: str) -> str:
     return file
 
 
-def imagery(
-    # plot: bool = False,
-    # region= None,
+def ice_vel(
+    plot: bool = False,
+    info: bool = False,
+    region=None,
+    spacing=10e3,
 ) -> xr.DataArray:
+    """
+    MEaSUREs Phase-Based Antarctica Ice Velocity Map, version 1:
+    https://nsidc.org/data/nsidc-0754/versions/1#anchor-1
+    Data part of https://doi.org/10.1029/2019GL083826
+
+    Parameters
+    ----------
+    plot : bool, optional
+        choose to plot grid, by default False
+    info : bool, optional
+        choose to print info on grid, by default False
+    region : str or np.ndarray, optional
+        GMT-format region to clip the loaded grid to, by default doesn't clip
+    spacing : str or int, optional
+        grid spacing to resample the loaded grid to, by default 10e3, original spacing
+        is 450m
+
+    Returns
+    -------
+    xr.DataArray
+        Returns a calculated grid of ice velocity in meters/year.
+    """
+
+    def velocity_calculation(fname, action, pooch):
+        "Load the .nc file, calculate velocity magnitude, save it back"
+        fname = Path(fname)
+        # Rename to the file to ***_magnitude.nc
+        fname_processed = fname.with_stem(fname.stem + "_magnitude")
+        # Only recalculate if new download or the processed file doesn't exist yet
+        if action in ("download", "update") or not fname_processed.exists():
+            grid = xr.load_dataset(fname)
+            processed = (grid.VX**2 + grid.VY**2) ** 0.5
+            # Save to disk
+            processed.to_netcdf(fname_processed)
+        return str(fname_processed)
+
+    # This is the path to the processed (magnitude) grid
+    path = pooch.retrieve(
+        url="https://n5eil01u.ecs.nsidc.org/MEASURES/NSIDC-0754.001/1996.01.01/antarctic_ice_vel_phase_map_v01.nc",  # noqa
+        downloader=EarthDataDownloader(),
+        known_hash=None,
+        progressbar=True,
+        processor=velocity_calculation,
+    )
+
+    grd = xr.load_dataarray(path)
+
+    # if region not set, use original, determined from utils.get_grid_info(path)
+    if region is None:
+        region = [-2800000.0, 2799800.0, -2799800.0, 2800000.0]
+
+    if (spacing == 450) and (region == [-2800000.0, 2799800.0, -2799800.0, 2800000.0]):
+        # print('spacing and region same as original, no processing')
+        pass
+
+    elif (spacing == 450) and (
+        region != [-2800000.0, 2799800.0, -2799800.0, 2800000.0]
+    ):
+        # print('spacing same as original, extracting new region ')
+        grd = pygmt.grdcut(grid=grd, region=region, verbose="q")
+
+    elif spacing < 450:
+        # print('spacing smaller than original, resampling')
+        grd = pygmt.grdsample(grid=grd, spacing=spacing, region=region, verbose="q")
+
+    elif spacing > 450:
+        # print('spacing larger than original, filtering and resampling')
+        grd = pygmt.grdfilter(
+            grid=grd,
+            filter=f"g{spacing}",
+            spacing=spacing,
+            region=region,
+            distance="0",
+            nans="r",
+            verbose="q",
+        )
+
+    if plot is True:
+        grd.plot(robust=True)
+    if info is True:
+        print(pygmt.grdinfo(grd))
+
+    return grd
+
+
+def modis_moa(
+    version: int = 750,
+) -> str:
+    """
+    Load the MODIS MoA imagery in either 750m or 125m resolutions.
+
+    Parameters
+    ----------
+    version : int, optional
+        choose between 750m or 125m resolutions, by default 750m
+
+    Returns
+    -------
+    str
+       filepath for either 750m or 125m MODIS MoA Imagery
+    """
+    if version == 125:
+        path = pooch.retrieve(
+            url="https://daacdata.apps.nsidc.org/pub/DATASETS/nsidc0593_moa2009_v02/geotiff/moa125_2009_hp1_v02.0.tif.gz",  # noqa
+            downloader=EarthDataDownloader(),
+            processor=pooch.Decompress(method="gzip", name="moa125_2009_hp1_v02.0.tif"),
+            known_hash=None,
+            progressbar=True,
+        )
+    elif version == 750:
+        path = pooch.retrieve(
+            url="https://daacdata.apps.nsidc.org/pub/DATASETS/nsidc0593_moa2009_v02/geotiff/moa750_2009_hp1_v02.0.tif.gz",  # noqa
+            downloader=EarthDataDownloader(),
+            processor=pooch.Decompress(method="gzip", name="moa750_2009_hp1_v02.0.tif"),
+            known_hash=None,
+            progressbar=True,
+        )
+
+    return path
+
+
+def imagery() -> xr.DataArray:
     """
     Load the file path of Antarctic imagery geotiff from LIMA:
     https://lima.usgs.gov/fullcontinent.php
@@ -57,7 +280,6 @@ def imagery(
         file path
     """
     path = pooch.retrieve(
-        # url="https://daacdata.apps.nsidc.org/pub/DATASETS/nsidc0730_MEASURES_MOA2014_v01/geotiff/moa750_2014_hp1_v01.tif", # noqa
         url="https://lima.usgs.gov/tiff_90pct.zip",
         processor=pooch.Unzip(),
         known_hash=None,
@@ -65,61 +287,6 @@ def imagery(
     )
     image = [p for p in path if p.endswith(".tif")][0]
 
-    # if region is not None:
-    #     grd = pygmt.grdcut(
-    #         grid=image,
-    #         region=region,
-    #         verbose='q')
-
-    # grd = rioxarray.open_rasterio(image)
-
-    # if region is not None:
-    #     grd = grd.rio.clip_box(
-    #         minx=region[0],
-    #         maxx=region[1],
-    #         miny=region[2],
-    #         maxy=region[3],
-    #         )
-
-    # spacing = kwargs.get('spacing', None)
-
-    # region = utils.get_grid_info(image)[1]
-    # spacing = utils.get_grid_info(image)[0]
-
-    # if region is not None and spacing is None:
-    #     print('using input region with grdcut')
-    #     grd = pygmt.grdcut(
-    #         grid=image,
-    #         region=region,
-    #         verbose='q')
-    # elif spacing is not None and region is None:
-    #     print('using input spacing')
-    #     grd = pygmt.grdfilter(
-    #         grid=image,
-    #         filter=f"g{spacing}",
-    #         spacing=spacing,
-    #         region=utils.get_grid_info(image)[1],
-    #         distance="0",
-    #         nans="r",
-    #         verbose="q",
-    #     )
-    # elif region and spacing is not None:
-    #     print('using input region and spacing')
-    #     grd = pygmt.grdfilter(
-    #         grid=image,
-    #         filter=f"g{spacing}",
-    #         spacing=spacing,
-    #         region=region,
-    #         distance="0",
-    #         nans="r",
-    #         verbose="q",
-    #     )
-    # else:
-    #     grd = image
-
-    # if plot is True:
-    #     grd.plot.imshow()
-
     return image
 
 
@@ -220,11 +387,18 @@ def bedmachine(
 
     if region is None:
         region = (-2800e3, 2800e3, -2800e3, 2800e3)
+
     path = pooch.retrieve(
-        url="https://storage.googleapis.com/ldeo-glaciology/bedmachine/BedMachineAntarctica_2019-11-05_v01.nc",  # noqa
+        url="https://n5eil01u.ecs.nsidc.org/MEASURES/NSIDC-0756.002/1970.01.01/BedMachineAntarctica_2020-07-15_v02.nc",  # noqa
+        downloader=EarthDataDownloader(),
         known_hash=None,
         progressbar=True,
     )
+    # path = pooch.retrieve(
+    #     url="https://storage.googleapis.com/ldeo-glaciology/bedmachine/BedMachineAntarctica_2019-11-05_v01.nc",  # noqa
+    #     known_hash=None,
+    #     progressbar=True,
+    # )
 
     if layer == "icebase":
         surface = pygmt.grdfilter(
@@ -234,6 +408,7 @@ def bedmachine(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
         thickness = pygmt.grdfilter(
@@ -243,6 +418,7 @@ def bedmachine(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
         grd = surface - thickness
@@ -255,6 +431,7 @@ def bedmachine(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
 
@@ -266,6 +443,7 @@ def bedmachine(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
         grd = grd + geoid
@@ -333,6 +511,7 @@ def bedmap2(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
         thickness_file = [p for p in path if p.endswith("thickness.tif")][0]
@@ -343,6 +522,7 @@ def bedmap2(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
         grd = surface - thickness
@@ -359,6 +539,7 @@ def bedmap2(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
 
@@ -371,6 +552,7 @@ def bedmap2(
             region=region,
             distance="0",
             nans="r",
+            registration="p",
             verbose="q",
         )
         grd = grd + geoid
@@ -659,8 +841,7 @@ def geothermal(
     xr.DataArray
          Returns a loaded, and optional clip/resampled grid of GHF data.
     """
-    # if region is None:
-    #     region = (-3330000, 3330000, -3330000, 3330000)
+
     if version == "burton-johnson-2020":
         path = pooch.retrieve(
             url="https://doi.org/10.5194/tc-14-3843-2020-supplement",
@@ -670,28 +851,28 @@ def geothermal(
             ),
             progressbar=True,
         )
+        file = [p for p in path if p.endswith("Mean.tif")][0]
         try:
             os.rename(
-                "C:\\Users\\matthewt\\AppData\\Local\\pooch\\pooch\\Cache\\Burton_Johnson_2020\\Geophysical GHF Summary Statistics Maps",  # noqa
-                "C:\\Users\\matthewt\\AppData\\Local\\pooch\\pooch\\Cache\\Burton_Johnson_2020\\Geophysical_GHF_Summary_Statistics_Maps",  # noqa
+                file[:-9],
+                file[:-9].replace(" ", "_"),
             )
         except FileNotFoundError:
             pass
 
-        file = [p for p in path if p.endswith("Mean.tif")][0]
-        if region is None:
-            region = utils.get_grid_info(file)[1]
-        if spacing is None:
-            spacing = utils.get_grid_info(file)[0]
+        file = glob.glob(f"{pooch.os_cache('pooch')}/Burton_Johnson_2020/**/*mean.tif")[
+            0
+        ]  # noqa
 
-        grd = pygmt.grdfilter(
-            grid=file,
-            filter=f"g{spacing}",
-            spacing=spacing,
-            region=region,
-            distance="0",
-            nans="r",
-            verbose="q",
+        grid = xr.load_dataarray(file)
+        grid = grid.squeeze()
+
+        # found from utils.get_grid_info(grid)
+        initial_spacing = 17000
+        initial_region = [-2543500.0, 2624500.0, -2121500.0, 2213500.0]
+
+        resampled = resample_grid(
+            grid, initial_spacing, initial_region, spacing, region
         )
 
     elif version == "losing-ebbing-2021":
@@ -705,19 +886,22 @@ def geothermal(
         df["x"], df["y"] = transformer.transform(df.Lat.tolist(), df.Lon.tolist())
 
         if region is None:
-            region = (-2800e3, 2800e3, -2800e3, 2800e3)
+            region = regions.antarctica
         if spacing is None:
             spacing = 20e3
 
         df = pygmt.blockmedian(
-            df[["x", "y", "HF [mW/m2]"]], spacing=spacing, region=region, verbose="q"
+            df[["x", "y", "HF [mW/m2]"]],
+            spacing=spacing,
+            region=region,
+            #  verbose="q",
         )
-        grd = pygmt.surface(
+        resampled = pygmt.surface(
             data=df[["x", "y", "HF [mW/m2]"]],
             spacing=spacing,
             region=region,
-            M="2c",
-            verbose="q",
+            # M="1c",
+            # verbose="q",
         )
     elif version == "aq1":
         path = pooch.retrieve(
@@ -726,26 +910,19 @@ def geothermal(
             progressbar=True,
         )
         file = xr.load_dataset(path)["Q"]
-        if region is None:
-            region = utils.get_grid_info(file)[1]
-        if spacing is None:
-            spacing = utils.get_grid_info(file)[0]
 
-        grd = pygmt.grdfilter(
-            grid=file,
-            filter=f"g{spacing}",
-            spacing=spacing,
-            region=region,
-            distance="0",
-            nans="r",
-            verbose="q",
-        )
-        grd = grd * 1000
+        # found from utils.get_grid_info(file)
+        initial_spacing = 20071.6845878
+        initial_region = [-2800000.0, 2800000.0, -2800000.0, 2800000.0]
+
+        grid = resample_grid(file, initial_spacing, initial_region, spacing, region)
+
+        resampled = grid * 1000
 
     else:
         print("invalid version string")
     if plot is True:
-        grd.plot(robust=True)
+        resampled.plot(robust=True)
     if info is True:
-        print(pygmt.grdinfo(grd))
-    return grd
+        print(pygmt.grdinfo(resampled))
+    return resampled
