@@ -16,7 +16,8 @@ import pyogrio
 import verde as vd
 import xarray as xr
 from pyproj import Transformer
-import rioxarray
+
+from antarctic_plots import maps
 
 if TYPE_CHECKING:
     import geopandas as gpd
@@ -42,20 +43,36 @@ def get_grid_info(grid):
         grid registration)
     """
 
-    spacing = pygmt.grdinfo(grid, per_column="n", o=7)[:-1]
-    region = [float(pygmt.grdinfo(grid, per_column="n", o=i)[:-1]) for i in range(4)]
-    zmin = float(pygmt.grdinfo(grid, per_column="n", o=4)[:-1])
-    zmax = float(pygmt.grdinfo(grid, per_column="n", o=5)[:-1])
-
     if isinstance(grid, str):
+        # grid = xr.load_dataarray(grid)
         try:
-            grid = pygmt.load_dataarray(grid)
+            # grid = pygmt.load_dataarray(grid)
+            grid = xr.load_dataarray(grid)
         except ValueError:
-            grid = xr.open_rasterio(grid)
+            print("getting grid info didnt work")
+            pass
+            # grid = xr.open_rasterio(grid)
+            # grid = rioxarray.open_rasterio(grid)
 
-    reg = grid.gmt.registration
+    if int(len(grid.dims)) > 2:
+        grid = grid.squeeze()
 
-    registration = "g" if reg == 0 else "p"
+    try:
+        spacing = pygmt.grdinfo(grid, per_column="n", o=7)[:-1]
+        region = [
+            float(pygmt.grdinfo(grid, per_column="n", o=i)[:-1]) for i in range(4)
+        ]
+        zmin = float(pygmt.grdinfo(grid, per_column="n", o=4)[:-1])
+        zmax = float(pygmt.grdinfo(grid, per_column="n", o=5)[:-1])
+        reg = grid.gmt.registration
+        registration = "g" if reg == 0 else "p"
+    except Exception:
+        print("grid info can't be extracted, check number of dimensions, should be 2.")
+        spacing = None
+        region = None
+        zmin = None
+        zmax = None
+        registration = None
 
     return spacing, region, zmin, zmax, registration
 
@@ -111,16 +128,21 @@ def latlon_to_epsg3031(
         [e, w, n, s]
     """
     transformer = Transformer.from_crs("epsg:4326", "epsg:3031")
-    df[output[0]], df[output[1]] = transformer.transform(
-        df[input[1]].tolist(), df[input[0]].tolist()
-    )
-    if reg is True:
-        df = [
-            df[output[0]].min(),
-            df[output[0]].max(),
-            df[output[1]].max(),
-            df[output[1]].min(),
-        ]
+
+    if isinstance(df, list):
+        ll = df.copy()
+        df = list(transformer.transform(ll[0], ll[1]))
+    else:
+        df[output[0]], df[output[1]] = transformer.transform(
+            df[input[1]].tolist(), df[input[0]].tolist()
+        )
+        if reg is True:
+            df = [
+                df[output[0]].min(),
+                df[output[0]].max(),
+                df[output[1]].max(),
+                df[output[1]].min(),
+            ]
     return df
 
 
@@ -131,8 +153,8 @@ def epsg3031_to_latlon(df, reg: bool = False, input=["x", "y"], output=["lon", "
 
     Parameters
     ----------
-    df : pd.DataFrame
-        input dataframe with easting and northing columns
+    df : pd.DataFrame or list
+        input dataframe with easting and northing columns, or list [x,y]
     reg : bool, optional
         if true, returns a GMT formatted region string, by default False
     input : list, optional
@@ -143,53 +165,120 @@ def epsg3031_to_latlon(df, reg: bool = False, input=["x", "y"], output=["lon", "
     Returns
     -------
     pd.DataFrame or np.ndarray
-        Updated dataframe with new latitude and longitude columns or np.ndarray in
-        format [e, w, n, s]
+        Updated dataframe with new latitude and longitude columns, np.ndarray in
+        format [e, w, n, s], or list in format [lat, lon]
     """
+
     transformer = Transformer.from_crs("epsg:3031", "epsg:4326")
-    df[output[1]], df[output[0]] = transformer.transform(
-        df[input[1]].tolist(), df[input[0]].tolist()
-    )
-    if reg is True:
-        df = [
-            df[output[0]].min(),
-            df[output[0]].max(),
-            df[output[1]].min(),
-            df[output[1]].max(),
-        ]
+
+    if isinstance(df, list):
+        xy = df.copy()
+        df = list(transformer.transform(xy[0], xy[1]))
+    else:
+        df[output[1]], df[output[0]] = transformer.transform(
+            df[input[0]].tolist(), df[input[1]].tolist()
+        )
+        if reg is True:
+            df = [
+                df[output[0]].min(),
+                df[output[0]].max(),
+                df[output[1]].min(),
+                df[output[1]].max(),
+            ]
     return df
 
 
-def reg_str_to_df(input, names=["x", "y"]):
+def reg_str_to_df(input, names=["x", "y"], reverse=False):
     """
     Convert GMT region string [e, w, n, s] to pandas dataframe with coordinates of
     region corners
 
     Parameters
     ----------
-    input : np.ndarray
-        Array of 4 strings in GMT format; [e, w, n, s]
+    input : np.ndarray or pd.DataFrame
+        Array of 4 strings in GMT format; [e, w, n, s], or, if `reverse` is True, a
+        DataFrame with easting and northing columns with names set by `names`
     names : list, optional
-        Names of names to use for easting and northing, by default ["x", "y"]
-
+        List of names to use for easting and northing, by default ["x", "y"]
+    reverse : bool, False
+        If True, convert from df to region string.
     Returns
     -------
-    pd.DataFrame
+    pd.DataFrame or np.ndarray
         Dataframe with easting and northing columns, and a row for each corner of the
-        region.
+        region, or, if `reverse` is True, a array in the format [e,w,n,s].
     """
-    bl = (input[0], input[2])
-    br = (input[1], input[2])
-    tl = (input[0], input[3])
-    tr = (input[1], input[3])
-    df = pd.DataFrame(data=[bl, br, tl, tr], columns=(names[0], names[1]))
+    if reverse is False:
+        bl = (input[0], input[2])
+        br = (input[1], input[2])
+        tl = (input[0], input[3])
+        tr = (input[1], input[3])
+        df = pd.DataFrame(data=[bl, br, tl, tr], columns=(names[0], names[1]))
+
+    elif reverse is True:
+        df = [
+            input[names[0]][0],
+            input[names[0]][1],
+            input[names[1]][0],
+            input[names[1]][2],
+        ]
+
     return df
 
 
-def GMT_reg_xy_to_ll(input):
+def GMT_reg_xy_to_ll(input, decimal_degree=False):
     """
     Convert GMT region string [e, w, n, s] in EPSG:3031 to deg:min:sec
 
+    Parameters
+    ----------
+    input : np.ndarray
+        Array of 4 strings in GMT format; [e, w, n, s] in meters
+    decimal_degrees: bool, False
+        if True, will return results as decimal degrees instead of deg:min:sec
+
+    Returns
+    -------
+    np.ndarray
+        Array of 4 strings in GMT format; [e, w, n, s] in lat, lon
+    """
+    df = reg_str_to_df(input)
+    df_proj = epsg3031_to_latlon(df, reg=True)
+
+    if decimal_degree is False:
+        output = [dd2dms(x) for x in df_proj]
+    elif decimal_degree is True:
+        output = df_proj
+    return output
+
+
+def GMT_reg_to_bounding_box(input):
+    """
+    Convert GMT region string [e, w, n, s] to bounding box format used for icepyx:
+    [ lower left long,
+      lower left lat,
+      upper right long,
+      uper right lat
+    ]
+
+    Parameters
+    ----------
+    input : np.ndarray
+        Array of 4 strings in GMT format; [e, w, n, s] in meters or degrees.
+
+    Returns
+    -------
+    np.ndarray
+        Array of 4 strings in bounding box format.
+    """
+    return [input[0], input[2], input[1], input[3]]
+
+
+def region_to_bounding_box(input):
+    """
+    Convert regions in format [e,w,n,s] in EPSG:3031 meters to format
+    [ll lon, ll lat, ur lon, ur lat] to be used as a bounding box in icepyx.
+
     Parameters
     ----------
     input : np.ndarray
@@ -198,12 +287,11 @@ def GMT_reg_xy_to_ll(input):
     Returns
     -------
     np.ndarray
-        Array of 4 strings in GMT format; [e, w, n, s] in lat, lon
+        Array of 4 strings in bounding box format.
     """
-    df = reg_str_to_df(input)
-    df_proj = epsg3031_to_latlon(df, reg=True)
-    output = [dd2dms(x) for x in df_proj]
-    return output
+    reg_ll = GMT_reg_xy_to_ll(input, decimal_degree=True)
+    box = GMT_reg_to_bounding_box(reg_ll)
+    return box
 
 
 def mask_from_shp(
@@ -222,7 +310,7 @@ def mask_from_shp(
     Parameters
     ----------
     shapefile : Union[str or gpd.geodataframe.GeoDataFrame]
-        either path to .shp filename, must by in same directory as accompanying files : 
+        either path to .shp filename, must by in same directory as accompanying files :
         .shx, .prj, .dbf, should be a closed polygon file, or shapefile which as already
          been loaded into a geodataframe.
     invert : bool, optional
@@ -250,7 +338,7 @@ def mask_from_shp(
     xarray.DataArray
         Returns either a masked grid, or the mask grid itself.
     """
-    
+
     if isinstance(shapefile, str):
         # shp = gpd.read_file(shapefile).geometry
         shp = pyogrio.read_dataframe(shapefile)
@@ -312,10 +400,15 @@ def alter_region(
     list
         Returns of list of the following variables (region, buffer_region, proj)
     """
-    e = starting_region[0] + zoom + w_shift
-    w = starting_region[1] - zoom + w_shift
-    n = starting_region[2] + zoom - n_shift
-    s = starting_region[3] - zoom - n_shift
+    E, W = starting_region[0], starting_region[1]
+    N, S = starting_region[2], starting_region[3]
+
+    e = E + zoom + w_shift
+    w = W - zoom + w_shift
+
+    n = N + zoom - n_shift
+    s = S - zoom - n_shift
+
     region = [e, w, n, s]
 
     e_buff, w_buff, n_buff, s_buff = (
@@ -327,21 +420,14 @@ def alter_region(
 
     buffer_region = [e_buff, w_buff, n_buff, s_buff]
 
-    fig_height = 80
-    # fig_width = fig_height * (w - e) / (s - n)
-
-    ratio = (s - n) / (fig_height / 1000)
-
-    proj = f"x1:{ratio}"
-
     if print_reg is True:
         print(f"inner region is {int((w-e)/1e3)} x {int((s-n)/1e3)} km")
-    return region, buffer_region, proj
+    return region, buffer_region
 
 
 def set_proj(
     region: Union[str or np.ndarray],
-    fig_height: float = 10,
+    fig_height: float = 15,
 ) -> str:
     """
     Gives GMT format projection string from region and figure height.
@@ -384,7 +470,7 @@ def grd_trend(
     da : xr.DataArray
         input grid
     coords : list, optional
-        coordinate names of grid, by default ['x', 'y', 'z']
+        coordinate names of the supplied grid, by default ['x', 'y', 'z']
     deg : int, optional
         trend order to use, by default 1
     plot_all : bool, optional
@@ -397,6 +483,7 @@ def grd_trend(
     """
 
     df = vd.grid_to_table(da).astype("float64")
+    df.dropna(inplace=True)
     trend = vd.Trend(degree=deg).fit((df[coords[0]], df[coords[1]]), df[coords[2]])
     df["fit"] = trend.predict((df[coords[0]], df[coords[1]]))
     df["detrend"] = df[coords[2]] - df.fit
@@ -459,6 +546,8 @@ def grd_trend(
 def grd_compare(
     da1: Union[xr.DataArray, str],
     da2: Union[xr.DataArray, str],
+    plot: bool = False,
+    plot_type: str = "pygmt",
     **kwargs,
 ):
     """
@@ -471,7 +560,11 @@ def grd_compare(
         first grid, loaded grid of filename
     da2 : xr.DataArray or str
         second grid, loaded grid of filename
-
+    plot : bool, optional
+        plot the results, by default False
+    plot_type : str, optional
+        choose the style of plot, by default is pygmt, can choose xarray for faster,
+        simplier plots.
     Keyword Args
     ------------
     shp_mask : str
@@ -511,16 +604,42 @@ def grd_compare(
             "grid spacings and regions dont match, using smaller spacing",
             f"({spacing}m) and inner region.",
         )
-        grid1 = pygmt.grdsample(da1, spacing=spacing, region=region, registration="p")
-        grid2 = pygmt.grdsample(da2, spacing=spacing, region=region, registration="p")
+        grid1 = pygmt.grdsample(
+            da1,
+            spacing=spacing,
+            region=region,
+            registration="p",
+            verbose="e",
+        )
+        grid2 = pygmt.grdsample(
+            da2,
+            spacing=spacing,
+            region=region,
+            registration="p",
+            verbose="e",
+        )
     elif da1_spacing != da2_spacing:
         print("grid spacings dont match, using smaller spacing of supplied grids")
-        grid1 = pygmt.grdsample(da1, spacing=spacing, registration="p")
-        grid2 = pygmt.grdsample(da2, spacing=spacing, registration="p")
+        grid1 = pygmt.grdsample(
+            da1,
+            spacing=spacing,
+            registration="p",
+            verbose="e",
+        )
+        grid2 = pygmt.grdsample(
+            da2,
+            spacing=spacing,
+            registration="p",
+            verbose="e",
+        )
     elif da1_reg != da2_reg:
         print("grid regions dont match, using inner region of supplied grids")
-        grid1 = pygmt.grdcut(da1, region=region, registration="p")
-        grid2 = pygmt.grdcut(da2, region=region, registration="p")
+        grid1 = pygmt.grdcut(
+            da1,
+            region=region,
+            verbose="e",
+        )
+        grid2 = pygmt.grdcut(da2, region=region, verbose="e")
     else:
         print("grid regions and spacing match")
         grid1 = da1
@@ -528,47 +647,84 @@ def grd_compare(
 
     dif = grid1 - grid2
 
-    vmin = min((np.nanmin(da1), np.nanmin(da2)))
-    vmax = max((np.nanmax(da1), np.nanmax(da2)))
+    # get individual grid min/max values (and masked values if shapefile is provided)
+    grid1_cpt_lims = get_min_max(grid1, shp_mask)
+    grid2_cpt_lims = get_min_max(grid2, shp_mask)
+    diff_cpt_lims = get_min_max(dif, shp_mask)
 
-    if kwargs.get("robust", False) is True:
-        vmin, vmax = None, None
-    if shp_mask is not None:
-        masked1 = mask_from_shp(shp_mask, xr_grid=grid1, masked=True, invert=False)
-        masked2 = mask_from_shp(shp_mask, xr_grid=grid2, masked=True, invert=False)
-        vmin = min((np.nanmin(masked1), np.nanmin(masked2)))
-        vmax = max((np.nanmax(masked1), np.nanmax(masked2)))
+    # get min and max of both grids together
+    vmin = min((grid1_cpt_lims[0], grid2_cpt_lims[0]))
+    vmax = max(grid1_cpt_lims[1], grid2_cpt_lims[1])
 
-    fig, ax = plt.subplots(ncols=3, nrows=1, figsize=(20, 20))
-    grid1.plot(
-        ax=ax[0],
-        cmap="viridis",
-        vmin=vmin,
-        vmax=vmax,
-        robust=True,
-        cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
-    )
-    grid2.plot(
-        ax=ax[1],
-        cmap="viridis",
-        vmin=vmin,
-        vmax=vmax,
-        robust=True,
-        cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
-    )
-    dif.plot(
-        ax=ax[2],
-        robust=True,
-        cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
-    )
-    for a in ax:
-        a.set_xticklabels([])
-        a.set_yticklabels([])
-        a.set_xlabel("")
-        a.set_ylabel("")
-        a.set_aspect("equal")
+    if plot is True:
+        if plot_type == "pygmt":
+            fig = maps.plot_grd(
+                grid1,
+                cmap="viridis",
+                plot_regin=region,
+                coast=True,
+                cbar_label=kwargs.get("grid1_name", "grid 1"),
+                cpt_lims=(vmin, vmax),
+            )
+            fig = maps.plot_grd(
+                grid2,
+                cmap="viridis",
+                plot_regin=region,
+                coast=True,
+                cbar_label=kwargs.get("grid2_name", "grid 2"),
+                cpt_lims=(vmin, vmax),
+                origin_shift="xshift",
+                fig=fig,
+            )
+            fig = maps.plot_grd(
+                dif,
+                cmap="polar",
+                plot_regin=region,
+                coast=True,
+                cbar_label="difference",
+                cpt_lims=diff_cpt_lims,
+                origin_shift="xshift",
+                fig=fig,
+            )
+            fig.show()
 
-    return dif
+        elif plot_type == "xarray":
+            if kwargs.get("robust", False) is True:
+                vmin, vmax = None, None
+
+            fig, ax = plt.subplots(ncols=3, nrows=1, figsize=(20, 20))
+            grid1.plot(
+                ax=ax[0],
+                cmap="viridis",
+                vmin=vmin,
+                vmax=vmax,
+                robust=True,
+                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
+            )
+            grid2.plot(
+                ax=ax[1],
+                cmap="viridis",
+                vmin=vmin,
+                vmax=vmax,
+                robust=True,
+                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
+            )
+            dif.plot(
+                ax=ax[2],
+                vmin=diff_cpt_lims[0],
+                vmax=diff_cpt_lims[1],
+                robust=True,
+                cmap="RdBu_r",
+                cbar_kwargs={"orientation": "horizontal", "anchor": (1, 1.8)},
+            )
+            for a in ax:
+                a.set_xticklabels([])
+                a.set_yticklabels([])
+                a.set_xlabel("")
+                a.set_ylabel("")
+                a.set_aspect("equal")
+
+    return dif, grid1, grid2
 
 
 def make_grid(
@@ -849,3 +1005,160 @@ def coherency(grids: list, label: str, **kwargs):
             label='3'
             )
     """
+
+
+def square_subplots(n):
+    """
+    From https://github.com/matplotlib/grid-strategy/blob/master/src/grid_strategy/strategies.py # noqa
+    Return an arrangement of rows containing ``n`` axes that is as close to
+    square as looks good.
+    :param n:
+        The number of plots in the subplot
+    :return:
+        Returns a  :class:`tuple` of length ``nrows``, where each element
+        represents the number of plots in that row, so for example a 3 x 2
+        grid would be represented as ``(3, 3)``, because there are 2 rows
+        of length 3.
+    Example:
+    --------
+    .. code::
+        >>> GridStrategy.get_grid(7)
+        (2, 3, 2)
+        >>> GridStrategy.get_grid(6)
+        (3, 3)
+    """
+    SPECIAL_CASES = {3: (2, 1), 5: (2, 3)}
+    if n in SPECIAL_CASES:
+        return SPECIAL_CASES[n]
+
+    # May not work for very large n
+    n_sqrtf = np.sqrt(n)
+    n_sqrt = int(np.ceil(n_sqrtf))
+
+    if n_sqrtf == n_sqrt:
+        # Perfect square, we're done
+        x, y = n_sqrt, n_sqrt
+    elif n <= n_sqrt * (n_sqrt - 1):
+        # An n_sqrt x n_sqrt - 1 grid is close enough to look pretty
+        # square, so if n is less than that value, will use that rather
+        # than jumping all the way to a square grid.
+        x, y = n_sqrt, n_sqrt - 1
+    elif not (n_sqrt % 2) and n % 2:
+        # If the square root is even and the number of axes is odd, in
+        # order to keep the arrangement horizontally symmetrical, using a
+        # grid of size (n_sqrt + 1 x n_sqrt - 1) looks best and guarantees
+        # symmetry.
+        x, y = (n_sqrt + 1, n_sqrt - 1)
+    else:
+        # It's not a perfect square, but a square grid is best
+        x, y = n_sqrt, n_sqrt
+
+    if n == x * y:
+        # There are no deficient rows, so we can just return from here
+        return tuple(x for i in range(y))
+
+    # If exactly one of these is odd, make it the rows
+    if (x % 2) != (y % 2) and (x % 2):
+        x, y = y, x
+
+    return x, y
+
+
+def random_color():
+    """
+    generate a random color in format R/G/B
+
+    Returns
+    -------
+    str
+        returns a random color string
+    """
+    color = (
+        f"{int(np.random.random() * 256)}/{int(np.random.random() * 256)}"
+        f"/{int(np.random.random() * 256)}"
+    )
+    return color
+
+
+def get_min_max(
+    grid: xr.DataArray,
+    shapefile: Union[str or gpd.geodataframe.GeoDataFrame] = None,
+):
+    """
+    Get a grids max and min values, optionally just for the region within a shapefile.
+
+    Parameters
+    ----------
+    grid : xr.DataArray
+        grid to get values for
+    shapefile : Union[str or gpd.geodataframe.GeoDataFrame], optional
+        path or loaded shapefile to use for a mask, by default None
+
+    Returns
+    -------
+    tuple
+        returns the min and max values.
+    """
+
+    if shapefile is None:
+        v_min, v_max = np.nanmin(grid), np.nanmax(grid)
+
+    elif shapefile is not None:
+        masked = mask_from_shp(shapefile, xr_grid=grid, masked=True, invert=False)
+        v_min, v_max = np.nanmin(masked), np.nanmax(masked)
+
+    return (v_min, v_max)
+
+
+def shapes_to_df(shapes: list):
+    """
+    convert the output of `regions.draw_region` and `profile.draw_lines` to a dataframe
+    of x and y points
+
+    Parameters
+    ----------
+    shapes : list
+        list of vertices
+
+    Returns
+    -------
+    pd.DataFrame
+        Dataframe with x, y, and shape_num.
+    """
+
+    df = pd.DataFrame()
+    for i, j in enumerate(shapes):
+        lon = [coord[0] for coord in j]
+        lat = [coord[1] for coord in j]
+        shape = pd.DataFrame({"lon": lon, "lat": lat, "shape_num": i})
+        df = pd.concat((df, shape))
+
+    df_xy = latlon_to_epsg3031(df)
+
+    return df_xy
+
+
+def polygon_to_region(polygon: list):
+    """
+    convert the output of `regions.draw_region` to bounding region in EPSG:3031
+
+    Parameters
+    ----------
+    polyon : list
+        list of polygon vertices
+
+    Returns
+    -------
+    list
+        list in format [e,w,n,s]
+    """
+
+    df = shapes_to_df(polygon)
+
+    if df.shape_num.max() > 0:
+        print("supplied dataframe has multiple polygons, only using the first one.")
+        df = df[df.shape_num == 0]
+
+    region = vd.get_region((df.x, df.y))
+
+    return region
